@@ -108,7 +108,8 @@ sub parse {
 	my @witnesses; # Keep the ordered list of our witnesses
     my %ac_wits;  # Track these for later removal
     foreach my $sigil ( map { $_->{'witness'} } @{$table->{'alignment'}} ) {
-        my $wit = $tradition->add_witness( 'sigil' => $sigil );
+        my $wit = $tradition->add_witness( 
+        	'sigil' => $sigil, 'sourcetype' => 'collation' );
         $wit->path( [ $c->start ] );
         push( @witnesses, $wit );
         my $aclabel = $c->ac_label;
@@ -206,7 +207,7 @@ sub make_nodes {
 			if( exists( $unique{$word} ) ) {
 				$rdg = $unique{$word};
 			} else {
-				my %args = ( 'id' => join( ',', $idx, $j+1 ),
+				my %args = ( 'id' => 'r' . join( '.', $idx, $j+1 ),
 					'rank' => $idx,
 					'text' => $word,
 					'collation' => $c );
