@@ -95,11 +95,9 @@ source of the XML to be parsed.
 
 =begin testing
 
-use File::Temp;
 use Safe::Isa;
 use Test::Warn;
 use Text::Tradition;
-use Text::Tradition::Directory;
 use TryCatch;
 binmode STDOUT, ":utf8";
 binmode STDERR, ":utf8";
@@ -121,13 +119,11 @@ if( $t ) {
 
 # TODO add a relationship, add a stemma, write graphml, reparse it, check that 
 # the new data is there
-$t->language('Greek');
-my $stemma_enabled;
-try {
-	$stemma_enabled = $t->enable_stemmata;
-} catch {
-	ok( 1, "Skipping stemma tests without Analysis module" );
+my $language_enabled = $t->can('language');
+if( $language_enabled ) {
+	$t->language('Greek');
 }
+my $stemma_enabled = $t->can('add_stemma');
 if( $stemma_enabled ) {
 	$t->add_stemma( 'dotfile' => 't/data/florilegium.dot' );
 }
@@ -144,7 +140,9 @@ if( $newt ) {
     is( scalar $newt->collation->paths, 376, "Collation has all paths" );
     is( scalar $newt->witnesses, 13, "Collation has all witnesses" );
     is( scalar $newt->collation->relationships, 1, "Collation has added relationship" );
-    is( $newt->language, 'Greek', "Tradition has correct language setting" );
+    if( $language_enabled ) {
+	    is( $newt->language, 'Greek', "Tradition has correct language setting" );
+	}
     my $rel = $newt->collation->get_relationship( 'w12', 'w13' );
     ok( $rel, "Found set relationship" );
     is( $rel->annotation, 'This is some note', "Relationship has its properties" );
@@ -154,28 +152,6 @@ if( $newt ) {
     }
 }
 
-# Test user save / restore
-my $fh = File::Temp->new();
-my $file = $fh->filename;
-$fh->close;
-my $dsn = "dbi:SQLite:dbname=$file";
-my $userstore = Text::Tradition::Directory->new( { dsn => $dsn,
-	extra_args => { create => 1 } } );
-my $scope = $userstore->new_scope();
-my $testuser = $userstore->create_user( { url => 'http://example.com' } );
-ok( $testuser->$_isa('Text::Tradition::User'), "Created test user via userstore" );
-$testuser->add_tradition( $newt );
-is( $newt->user->id, $testuser->id, "Assigned tradition to test user" );
-$graphml_str = $newt->collation->as_graphml;
-my $usert;
-warning_is {
-	$usert = Text::Tradition->new( 'input' => 'Self', 'string' => $graphml_str );
-} 'DROPPING user assignment without a specified userstore',
-	"Got expected user drop warning on parse";
-$usert = Text::Tradition->new( 'input' => 'Self', 'string' => $graphml_str,
-	'userstore' => $userstore );
-is( $usert->user->id, $testuser->id, "Parsed tradition with userstore points to correct user" );
-
 # Test warning if we can
 unless( $stemma_enabled ) {
 	my $nst;
@@ -203,25 +179,19 @@ sub parse {
     $tradition->name( $graph_data->{'name'} );
 
     my $use_version;
-    my $tmeta = $tradition->meta;
-    my $cmeta = $collation->meta;
     foreach my $gkey ( keys %{$graph_data->{'global'}} ) {
 		my $val = $graph_data->{'global'}->{$gkey};
 		if( $gkey eq 'version' ) {
 			$use_version = $val;
 		} elsif( $gkey eq 'stemmata' ) {
 			# Make sure we can handle stemmata
-			my $stemma_enabled;
-			try {
-				$stemma_enabled = $tradition->enable_stemmata;
-			} catch {
-				warn "Analysis module not installed; DROPPING stemmata";
-			}
 			# Parse the stemmata into objects
-			if( $stemma_enabled ) {
+			if( $tradition->can('add_stemma') ) {
 				foreach my $dotstr ( split( /\n/, $val ) ) {
 					$tradition->add_stemma( 'dot' => $dotstr );
 				}
+			} else {
+				warn "Analysis module not installed; DROPPING stemmata";
 			}
 		} elsif( $gkey eq 'user' ) {
 			# Assign the tradition to the user if we can
@@ -236,10 +206,12 @@ sub parse {
 			} else {
 				warn( "DROPPING user assignment without a specified userstore" );
 			}
-		} elsif( $tmeta->has_attribute( $gkey ) ) {
+		} elsif( $tradition->can( $gkey ) ) {
 			$tradition->$gkey( $val );
-		} else {
+		} elsif( $collation->can( $gkey ) ) {
 			$collation->$gkey( $val );
+		} else {
+			warn( "DROPPING unsupported attribute $gkey" );
 		}
 	}
 		
@@ -249,7 +221,6 @@ sub parse {
     my %namechange = ( '#START#' => '__START__', '#END#' => '__END__' );
 
     # print STDERR "Adding collation readings\n";
-    my $need_morphology;
     foreach my $n ( @{$graph_data->{'nodes'}} ) {    	
     	# If it is the start or end node, we already have one, so
     	# grab the rank and go.
@@ -261,21 +232,11 @@ sub parse {
     		$collation->end->rank( $n->{'rank'} );
     		next;
     	}
-    	# HACKY but no better way yet
-    	# If $n has a 'lexemes' property then we will need the morphology for
-    	# the whole tradition.
-    	$need_morphology = 1 if exists $n->{'lexemes'};
 		my $gnode = $collation->add_reading( $n );
 		if( $gnode->id ne $n->{'id'} ) {
 			$namechange{$n->{'id'}} = $gnode->id;
 		}
     }
-    # HACK continued - if any of the readings had morphology info, we
-    # must enable it for the whole tradition. Just eval it, as we will
-    # have already been warned if the morphology extension isn't installed.
-    if( $need_morphology ) {
-    	eval { $tradition->enable_morphology };
-    }
         
     # Now add the edges.
     # print STDERR "Adding collation path edges\n";
